package com.ruoyi.system.utils;

import java.util.Comparator;
import java.nio.file.LinkOption;
import java.nio.file.FileVisitOption;
import java.util.zip.ZipOutputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.io.OutputStream;
import java.io.InputStream;
import java.net.URL;
import java.nio.file.OpenOption;
import java.nio.file.Files;
import java.nio.file.attribute.FileAttribute;
import java.nio.file.Paths;
import java.io.IOException;
import java.util.Map;
import com.ruoyi.common.core.utils.JwtUtils;
import java.util.HashMap;
import java.nio.file.Path;

public class ZipTool
{
    public static void main(final String[] args) {
        final String urlString = "https://file.movfen.com/sync-cloud/base/woo-synchronous-payment.zip";
        final Path newZipPath = generateWpPlugin(urlString, "https://wanxitrading.com", "https://wanxitrading.com", "woo-synchronous-payment-release");
        System.out.println("New zip file created at: " + newZipPath.toAbsolutePath());
    }
    
    public static Path generateWpPlugin(final String urlString, final String pluginDomain, final String backDomain, final String pluginName) {
        try {
            final Map<String, Object> claims = new HashMap<String, Object>();
            claims.put("domain", pluginDomain);
            claims.put("time", System.currentTimeMillis());
            final String key = JwtUtils.createToken((Map)claims);
            final String content = "<?php\n\nreturn [\n    'url' => '" + backDomain + "',\n    'key' => '" + key + "'\n];";
            return createAndProcessZip(urlString, "profile.php", content, pluginName);
        }
        catch (final IOException e) {
            e.printStackTrace();
            return null;
        }
    }
    
    public static Path createAndProcessZip(final String fileUrl, final String profileFileName, final String profileContent, final String pluginName) throws IOException {
        final String timestamp = System.currentTimeMillis() + "";
        final Path projectRoot = Paths.get("", new String[0]).toAbsolutePath();
        final Path tempDir = Files.createDirectories(projectRoot.resolve("tempDir_" + System.currentTimeMillis()), (FileAttribute<?>[])new FileAttribute[0]);
        final Path zipPath = tempDir.resolve("downloaded.zip");
        downloadFile(fileUrl, zipPath);
        final Path unzipDir = tempDir.resolve("unzipped");
        unzip(zipPath, unzipDir);
        final Path profilePath = unzipDir.resolve(pluginName + "/" + profileFileName);
        Files.write(profilePath, profileContent.getBytes(), new OpenOption[0]);
        final Path newZipPath = projectRoot.resolve("new-package_" + timestamp + ".zip");
        zipDirectory(unzipDir, newZipPath);
        deleteDirectory(tempDir);
        return newZipPath;
    }
    
    private static void downloadFile(final String fileUrl, final Path destination) throws IOException {
        try (final InputStream in = new URL(fileUrl).openStream();
             final OutputStream out = Files.newOutputStream(destination, new OpenOption[0])) {
            final byte[] buffer = new byte[1024];
            int bytesRead;
            while ((bytesRead = in.read(buffer)) != -1) {
                out.write(buffer, 0, bytesRead);
            }
        }
    }
    
    private static void unzip(final Path zipFile, final Path outputDir) throws IOException {
        try (final ZipInputStream zis = new ZipInputStream(Files.newInputStream(zipFile, new OpenOption[0]))) {
            ZipEntry entry;
            while ((entry = zis.getNextEntry()) != null) {
                final Path entryPath = outputDir.resolve(entry.getName());
                if (entry.isDirectory()) {
                    Files.createDirectories(entryPath, (FileAttribute<?>[])new FileAttribute[0]);
                }
                else {
                    Files.createDirectories(entryPath.getParent(), (FileAttribute<?>[])new FileAttribute[0]);
                    try (final OutputStream fos = Files.newOutputStream(entryPath, new OpenOption[0])) {
                        final byte[] buffer = new byte[1024];
                        int bytesRead;
                        while ((bytesRead = zis.read(buffer)) != -1) {
                            fos.write(buffer, 0, bytesRead);
                        }
                    }
                }
                zis.closeEntry();
            }
        }
    }
    
    private static void zipDirectory(final Path sourceDir, final Path zipFile) throws IOException {
        try (final ZipOutputStream zos = new ZipOutputStream(Files.newOutputStream(zipFile, new OpenOption[0]))) {
            Files.walk(sourceDir, new FileVisitOption[0]).filter(x$0 -> Files.isRegularFile(x$0, new LinkOption[0])).forEach(path -> {
                try {
                    final ZipEntry zipEntry = new ZipEntry(sourceDir.relativize(path).toString());
                    zos.putNextEntry(zipEntry);
                    Files.copy(path, zos);
                    zos.closeEntry();
                }
                catch (final IOException e) {
                    e.printStackTrace();
                }
                return;
            });
        }
    }
    
    public static void deleteDirectory(final Path path) throws IOException {
        Files.walk(path, new FileVisitOption[0]).sorted(Comparator.reverseOrder()).forEach(p -> {
            try {
                Files.delete(p);
            }
            catch (final IOException e) {
                e.printStackTrace();
            }
        });
    }
}
